@@ -1,6 +1,6 @@
 /* Desktop management for the Midnight Commander
  *
- * Copyright (C) 1998 The Free Software Foundation
+ * Copyright (C) 1998-1999 The Free Software Foundation
  *
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  *          Miguel de Icaza <miguel@nuclecu.unam.mx>
@@ -24,6 +24,7 @@
 #include "gmain.h"
 #include "gmetadata.h"
 #include "gdnd.h"
+#include "gmount.h"
 #include "gpopup.h"
 #include "../vfs/vfs.h"
 #include "main.h"
@@ -45,7 +46,7 @@ int desktop_auto_placement = FALSE;
 int desktop_snap_icons = FALSE;
 
 /* The computed name of the user's desktop directory */
-static char *desktop_directory;
+char *desktop_directory;
 
 /* Layout information:  number of rows/columns for the layout slots, and the array of slots.  Each
  * slot is an integer that specifies the number of icons that belong to that slot.
@@ -56,6 +57,9 @@ static int layout_cols;
 static int layout_rows;
 static struct layout_slot *layout_slots;
 
+
+int desktop_wm_is_gnome_compliant = -1;
+
 #define l_slots(u, v) (layout_slots[(u) * layout_rows + (v)])
 
 /* The last icon to be selected */
@@ -290,6 +294,18 @@ desktop_icon_info_place (DesktopIconInfo *dii, int xpos, int ypos)
 	g_free (filename);
 }
 
+/* Destroys the specified desktop icon */
+static void
+desktop_icon_info_destroy (DesktopIconInfo *dii)
+{
+	gtk_widget_destroy (dii->dicon);
+	remove_from_slot (dii);
+
+	g_free (dii->url);
+	g_free (dii->filename);
+	g_free (dii);
+}
+
 /* Destroys all the current desktop icons */
 static void
 destroy_desktop_icons (void)
@@ -356,8 +372,8 @@ typedef struct {
  * specified position if user_pos is TRUE.  If it is FALSE, the icons will be
  * auto-placed.
  */
-static void
-reload_desktop_icons (int user_pos, int xpos, int ypos)
+void
+desktop_reload_icons (int user_pos, int xpos, int ypos)
 {
 	struct dirent *dirent;
 	DIR *dir;
@@ -367,6 +383,7 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 	GSList *need_position_list, *sl;
 	GList *all_icons, *l;
 	char *desktop_url, *caption;
+	const char *mime;
 	int orig_xpos, orig_ypos;
 
 	dir = mc_opendir (desktop_directory);
@@ -406,7 +423,11 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 
 			full_name = g_concat_dir_and_file (desktop_directory, dirent->d_name);
 			fe = file_entry_from_file (full_name);
-			im = gicon_get_icon_for_file_speed (desktop_directory, fe, FALSE);
+			if (!fe){
+				g_free (full_name);
+				continue;
+			}
+			im = gicon_get_icon_for_file (desktop_directory, fe, FALSE);
 			file_entry_free (fe);
 			g_free (full_name);
 
@@ -431,8 +452,6 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 		if (have_pos) {
 			dii = desktop_icon_info_new (dirent->d_name, desktop_url, caption, x, y);
 			gtk_widget_show (dii->dicon);
-
-			g_free (full_name);
 		} else {
 			file_and_url_t *fau;
 
@@ -445,6 +464,8 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 			need_position_list = g_slist_prepend (need_position_list, fau);
 		}
 
+		g_free (full_name);
+		
 		if (desktop_url)
 			g_free (desktop_url);
 
@@ -463,7 +484,7 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 	for (l = all_icons; l; l = l->next) {
 		dii = l->data;
 		full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
-		gmeta_del_icon_pos (full_name);
+		gnome_metadata_delete (full_name);
 		g_free (full_name);
 		desktop_icon_info_destroy (dii);
 	}
@@ -480,7 +501,9 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 	orig_xpos = orig_ypos = 0;
 
 	for (sl = need_position_list; sl; sl = sl->next) {
-		file_and_url_t *fau = sl->data;
+		file_and_url_t *fau;
+
+		fau = sl->data;
 
 		if (user_pos && sl == need_position_list) {
 			/* If we are on the first icon, place it "by hand".
@@ -501,13 +524,46 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 			get_icon_auto_pos (&xpos, &ypos);
 		}
 
+		/* If the file dropped was a .desktop file, pull the suggested
+		 * title and icon from there
+		 */
+		mime = gnome_mime_type_or_default (fau->filename, NULL);
+		if (mime && strcmp (mime, "application/x-gnome-app-info") == 0) {
+			GnomeDesktopEntry *entry;
+			char *fullname;
+
+			fullname = g_concat_dir_and_file (desktop_directory, fau->filename);
+			entry = gnome_desktop_entry_load (fullname);
+			if (entry) {
+				if (entry->name) {
+					if (fau->caption)
+						g_free (fau->caption);
+
+					fau->caption = g_strdup (entry->name);
+					gnome_metadata_set (fullname, "icon-caption",
+							    strlen (fau->caption) + 1,
+							    fau->caption);
+				}
+
+				if (entry->icon)
+					gnome_metadata_set (fullname, "icon-filename",
+							    strlen (entry->icon) + 1,
+							    entry->icon);
+
+				gnome_desktop_entry_free (entry);
+			}
+			g_free (fullname);
+		}
+
 		dii = desktop_icon_info_new (fau->filename, fau->url, fau->caption, xpos, ypos);
 		gtk_widget_show (dii->dicon);
 
 		if (fau->url)
 			g_free (fau->url);
+
 		if (fau->caption)
 			g_free (fau->caption);
+
 		g_free (fau->filename);
 		g_free (fau);
 	}
@@ -520,8 +576,8 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 }
 
 /* Perform automatic arrangement of the desktop icons */
-static void
-arrange_desktop_icons (void)
+void
+desktop_arrange_icons (void)
 {
 	GList *icons, *l;
 	int xpos, ypos;
@@ -825,6 +881,168 @@ selection_stopped (GnomeIconTextItem *iti, gpointer data)
 	setup_editing_grab (dii);
 }
 
+static char *mount_known_locations [] = {
+	"/sbin/mount", "/bin/mount", "/etc/mount",
+	"/usr/sbin/mount", "/usr/etc/mount", "/usr/bin/mount",
+	NULL
+};
+
+static char *umount_known_locations [] = {
+	"/sbin/umount", "/bin/umount", "/etc/umount",
+	"/usr/sbin/umount", "/usr/etc/umount", "/usr/bin/umount",
+	NULL
+};
+
+/*
+ * Returns the full path to the mount command
+ */
+static char *
+find_command (char **known_locations)
+{
+	int i;
+
+	for (i = 0; known_locations [i]; i++){
+		if (g_file_exists (known_locations [i]))
+			return known_locations [i];
+	}
+	return NULL;
+}
+
+gboolean
+is_mountable (char *filename, file_entry *fe, int *is_mounted, char **point)
+{
+	char buffer [128], *p;
+	umode_t mode;
+	struct stat s;
+
+	if (point)
+		*point = NULL;
+	
+	if (!S_ISLNK (fe->buf.st_mode))
+		return FALSE;
+
+	if (stat (filename, &s) == -1)
+		return FALSE;
+	mode = s.st_mode;
+		
+	if (!S_ISBLK (mode))
+		return FALSE;
+
+	if (readlink (filename, buffer, sizeof (buffer)) == -1)
+		return FALSE;
+
+	p = is_block_device_mountable (buffer);
+	if (!p)
+		return FALSE;
+
+	if (point)
+		*point = p;
+	else
+		g_free (point);
+			
+	*is_mounted = is_block_device_mounted (buffer);
+		
+	return TRUE;
+}
+
+gboolean
+do_mount_umount (char *filename, gboolean is_mount)
+{
+	static char *mount_command;
+	static char *umount_command;
+	char *op;
+	char buffer [128];
+
+	if (is_mount){
+		if (!mount_command)
+			mount_command = find_command (mount_known_locations);
+		op = mount_command;
+	} else {
+		if (!umount_command)
+			umount_command = find_command (umount_known_locations);
+		op = umount_command;
+	}
+
+	if (readlink (filename, buffer, sizeof (buffer)) == -1)
+		return FALSE;
+	
+	if (op){
+		char *command;
+		FILE *f;
+		
+		command = g_strconcat (op, " ", buffer, NULL);
+		open_error_pipe ();
+		f = popen (command, "r");
+		if (f == NULL)
+			close_error_pipe (1, _("While running the mount/umount command"));
+		else 
+			close_error_pipe (0, 0);
+		pclose (f);
+		
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static char *eject_known_locations [] = {
+	"/usr/bin/eject",
+	"/sbin/eject",
+	"/bin/eject",
+	NULL
+};
+
+/*
+ * Returns whether the device is ejectable
+ *
+ * Right now the test only checks if this system has the eject
+ * command
+ */
+gboolean 
+is_ejectable (char *filename)
+{
+	if (find_command (eject_known_locations))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+/*
+ * Ejects the device pointed by filename
+ */
+gboolean
+do_eject (char *filename)
+{
+	char *eject_command = find_command (eject_known_locations);
+	char *command;
+	FILE *f;
+
+	if (!eject_command)
+		return FALSE;
+
+	command = g_strconcat (eject_command, " ", filename, NULL);
+	open_error_pipe ();
+	f = popen (command, "r");
+	if (f == NULL)
+		close_error_pipe (1, _("While running the eject command"));
+	else
+		close_error_pipe (0, 0);
+
+	pclose (f);
+
+	return TRUE;
+}
+
+static gboolean
+try_to_mount (char *filename, file_entry *fe)
+{
+	int x;
+	
+	if (!is_mountable (filename, fe, &x, NULL))
+		return FALSE;
+
+	return do_mount_umount (filename, TRUE);
+}
+
 /**
  * desktop_icon_info_open:
  * @dii: The desktop icon to open.
@@ -844,14 +1062,34 @@ desktop_icon_info_open (DesktopIconInfo *dii)
 	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
 
 	fe = file_entry_from_file (filename);
-
+	if (!fe){
+		message (1, _("Error"), "I could not fetch the information from the file");
+		return;
+	}
+	
 	if (S_ISDIR (fe->buf.st_mode) || link_isdir (fe))
 		new_panel_at (filename);
 	else {
-		if (is_exe (fe->buf.st_mode) && if_link_is_exe (fe))
-			my_system (EXECUTE_AS_SHELL, shell, filename);
-		else
-			gmc_open_filename (filename, NULL);
+		int is_mounted;
+		char *point;
+		int launch = FALSE;
+		
+		if (is_mountable (filename, fe, &is_mounted, &point)){
+			if (!is_mounted){
+				if (try_to_mount (filename, fe))
+					launch = TRUE;
+			} else
+				launch = TRUE;
+
+			if (launch)
+				new_panel_at (point);
+			g_free (point);
+		} else {
+			if (is_exe (fe->buf.st_mode) && if_link_is_exe (desktop_directory, fe))
+				my_system (EXECUTE_AS_SHELL, shell, filename);
+			else
+				gmc_open_filename (filename, NULL);
+		}
 	}
 
 	file_entry_free (fe);
@@ -884,7 +1122,7 @@ desktop_icon_info_delete (DesktopIconInfo *dii)
 				erase_file (ctx, full_name, &progress_count, &progress_bytes, TRUE);
 		}
 
-		gmeta_del_icon_pos (full_name);
+		gnome_metadata_delete (full_name);
 	}
 
 	g_free (full_name);
@@ -903,7 +1141,7 @@ do_popup_menu (DesktopIconInfo *dii, GdkEventButton *event)
 	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
 
 	if (gpopup_do_popup (event, NULL, dii, 0, filename) != -1)
-		reload_desktop_icons (FALSE, 0, 0);
+		desktop_reload_icons (FALSE, 0, 0);
 
 	g_free (filename);
 }
@@ -1218,7 +1456,7 @@ drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *sel
 static void
 drag_end (GtkWidget *widget, GdkDragContext *context, gpointer data)
 {
-	reload_desktop_icons (FALSE, 0, 0);
+	desktop_reload_icons (FALSE, 0, 0);
 }
 
 /* Set up a desktop icon as a DnD source */
@@ -1267,10 +1505,12 @@ icon_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, gu
 				       TRUE,
 				       source_widget != NULL,
 				       source_widget && is_desktop_icon,
+				       desktop_directory,
 				       fe,
 				       dii->selected);
 
 	gdk_drag_status (context, action, time);
+	file_entry_free (fe);
 	return TRUE;
 }
 
@@ -1376,11 +1616,12 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 
 		full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
 		fe = file_entry_from_file (full_name);
+		g_free (full_name);
 		if (!fe)
 			return; /* eeeek */
 
-		if (gdnd_perform_drop (context, data, fe, full_name))
-			reload_desktop_icons (FALSE, 0, 0);
+		if (gdnd_perform_drop (context, data, desktop_directory, fe))
+			desktop_reload_icons (FALSE, 0, 0);
 
 		file_entry_free (fe);
 	}
@@ -1420,7 +1661,9 @@ desktop_icon_info_new (char *filename, char *url, char *caption, int xpos, int y
 
 	full_name = g_concat_dir_and_file (desktop_directory, filename);
 	fe = file_entry_from_file (full_name);
-
+	if (!fe)
+		return NULL;
+	
 	dii = g_new (DesktopIconInfo, 1);
 	dii->x = 0;
 	dii->y = 0;
@@ -1432,10 +1675,11 @@ desktop_icon_info_new (char *filename, char *url, char *caption, int xpos, int y
 			caption = url;
 	} else {
 		dii->url = NULL;
-		caption = filename;
+		if (caption == NULL)
+			caption = filename;
 	}
 	
-	icon_im = gicon_get_icon_for_file_speed (desktop_directory, fe, FALSE);
+	icon_im = gicon_get_icon_for_file (desktop_directory, fe, FALSE);
 	dii->dicon = desktop_icon_new (icon_im, caption);
 	dii->filename = g_strdup (filename);
 	dii->selected = FALSE;
@@ -1529,24 +1773,6 @@ desktop_icon_update_url (DesktopIconInfo *dii)
 	g_free (fullname);
 }
 
-/**
- * desktop_icon_info_destroy:
- * @dii: The desktop icon to destroy
- *
- * Destroys the specified desktop icon.
- **/
-void
-desktop_icon_info_destroy (DesktopIconInfo *dii)
-{
-	gtk_widget_destroy (dii->dicon);
-	remove_from_slot (dii);
-
-	gnome_metadata_delete (dii->filename);
-	g_free (dii->url);
-	g_free (dii->filename);
-	g_free (dii);
-}
-
 /* Creates the layout information array */
 static void
 create_layout_info (void)
@@ -1558,28 +1784,8 @@ create_layout_info (void)
 	layout_slots = g_new0 (struct layout_slot, layout_cols * layout_rows);
 }
 
-static void
-setup_trashcan (char *desktop_dir)
-{
-	char *trashcan_dir;
-	char *trash_pix;
-	
-	trashcan_dir = g_concat_dir_and_file (desktop_directory, _("Trashcan"));
-	trash_pix = g_concat_dir_and_file (ICONDIR, "trash.xpm");
-		
-	if (!g_file_exists (trashcan_dir)){
-		mkdir (trashcan_dir, 0777);
-		gnome_metadata_set (
-			trashcan_dir, "icon-filename", strlen (trash_pix) + 1, trash_pix);
-	}
-	
-	g_free (trashcan_dir);
-	g_free (trash_pix);
-}
-
-/*
- * Check that the user's desktop directory exists, and if not, create
- * the default desktop setup.
+/* Check that the user's desktop directory exists, and if not, create the
+ * default desktop setup.
  */
 static void
 create_desktop_dir (void)
@@ -1602,13 +1808,14 @@ create_desktop_dir (void)
 		if (mc_symlink (gnome_user_home_dir, home_link_name) != 0) {
 			message (FALSE,
 				 _("Warning"),
-				 _("Could not symlink %s to %s; will not have initial desktop icons."),
+				 _("Could not symlink %s to %s; "
+				   "will not have initial desktop icons."),
 				 gnome_user_home_dir, home_link_name);
 		}
 		g_free (home_link_name);
-	}
 
-/*	setup_trashcan (desktop_directory); */
+		gmount_setup_devices (FALSE);
+	}
 }
 
 /* Property placed on target windows */
@@ -1628,7 +1835,7 @@ static gboolean
 setup_xdnd_proxy (guint32 xid, GdkWindow *proxy_window)
 {
 	GdkAtom xdnd_proxy_atom;
-	guint32 proxy_xid;
+	Window proxy_xid;
 	Atom type;
 	int format;
 	unsigned long nitems, after;
@@ -1716,7 +1923,7 @@ setup_xdnd_proxy (guint32 xid, GdkWindow *proxy_window)
 static void
 setup_motif_dnd_proxy (guint32 xid, GdkWindow *proxy_window)
 {
-	guint32 proxy_xid;
+	Window proxy_xid;
 	MotifDragReceiverInfo info;
 	Atom receiver_info_atom;
 	guint32 myint;
@@ -1759,6 +1966,7 @@ desktop_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 				       TRUE,
 				       source_widget != NULL,
 				       source_widget && is_desktop_icon,
+				       desktop_directory,
 				       NULL,
 				       FALSE);
 
@@ -1788,8 +1996,8 @@ desktop_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x,
 		if (!desktop_fe)
 			return; /* eeek */
 
-		if (gdnd_perform_drop (context, data, desktop_fe, desktop_directory))
-			reload_desktop_icons (TRUE, x, y);
+		if (gdnd_perform_drop (context, data, desktop_directory, desktop_fe))
+			desktop_reload_icons (TRUE, x, y);
 
 		file_entry_free (desktop_fe);
 	}
@@ -1896,19 +2104,44 @@ find_click_proxy_window (void)
 static void
 handle_arrange_icons (GtkWidget *widget, gpointer data)
 {
-	arrange_desktop_icons ();
+	desktop_arrange_icons ();
+}
+
+/* Callback for creating a new panel window */
+static void
+handle_new_window (GtkWidget *widget, gpointer data)
+{
+	new_panel_at (gnome_user_home_dir);
+}
+
+void
+desktop_rescan_devices (void)
+{
+	gmount_setup_devices (TRUE);
+	desktop_reload_icons (FALSE, 0, 0);
+}
+
+/* Callback for rescanning the mountable devices */
+static void
+handle_rescan_devices (GtkWidget *widget, gpointer data)
+{
+	desktop_rescan_devices ();
 }
 
 /* Callback for rescanning the desktop directory */
 static void
 handle_rescan_desktop (GtkWidget *widget, gpointer data)
 {
-	reload_desktop_icons (FALSE, 0, 0);
+	desktop_reload_icons (FALSE, 0, 0);
 }
 
 /* The popup menu for the desktop */
-static GnomeUIInfo desktop_popup_items[] = {
+GnomeUIInfo desktop_popup_items[] = {
 	GNOMEUIINFO_ITEM_NONE (N_("Arrange Icons"), NULL, handle_arrange_icons),
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_NONE (N_("Create New Window"), NULL, handle_new_window),
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan Mountable Devices"), NULL, handle_rescan_devices),
 	GNOMEUIINFO_ITEM_NONE (N_("Rescan Desktop"), NULL, handle_rescan_desktop),
 	GNOMEUIINFO_END
 };
@@ -2205,9 +2438,11 @@ setup_desktop_click_proxy_window (void)
 {
 	click_proxy_gdk_window = find_click_proxy_window ();
 	if (!click_proxy_gdk_window) {
+		desktop_wm_is_gnome_compliant = 0;
 		g_warning ("Root window clicks will not work as no GNOME-compliant window manager could be found!");
 		return;
 	}
+	desktop_wm_is_gnome_compliant = 1;	
 
 	/* Make the proxy window send events to the invisible proxy widget */
 	gdk_window_set_user_data (click_proxy_gdk_window, proxy_invisible);
@@ -2327,7 +2562,7 @@ desktop_init (void)
 	gicon_init ();
 	create_layout_info ();
 	create_desktop_dir ();
-	reload_desktop_icons (FALSE, 0, 0);
+	desktop_reload_icons (FALSE, 0, 0);
 
 	/* Create the proxy window and initialize all proxying stuff */
 
