@@ -88,18 +88,8 @@ get_other_index (void)
 	return UNDEFINED_INDEX;
 }
 
-void
-set_current_panel (int index)
-{
-	GList *p;
-	
-	for (p = containers; index; p = p->next)
-		index--;
-	current_panel_ptr = p->data; 
-}
-
-void
-set_new_current_panel (WPanel *panel)
+static void
+set_current_panel (WPanel *panel)
 {
 	GList *p;
 
@@ -277,6 +267,49 @@ run_cmd (void)
 	}
 }
 
+static void
+gnome_exit (void)
+{
+	GtkWidget *w;
+	int v;
+	
+	w = gnome_message_box_new (
+		_("Notice that if you choose to terminate the file manager, you will\n"
+		  "also terminate the GNOME desktop handler.\n\n"
+		  "Are you sure you want to quit?"),
+		GNOME_MESSAGE_BOX_WARNING,
+		GNOME_STOCK_BUTTON_YES,
+		GNOME_STOCK_BUTTON_NO,
+		NULL);
+	v = gnome_dialog_run (GNOME_DIALOG (w));
+	if (v != 0)
+		return;
+
+	w = gnome_message_box_new (
+		N_("The file manager and the desktop handler are now terminating\n\n"
+		   "If you want to start up again the desktop handler or the file manager\n"
+		   "you can launch it from the Panel, or you can run the UNIX command `gmc'\n\n"
+		   "Press OK to terminate the application, or cancel to continue using it."),
+		GNOME_MESSAGE_BOX_INFO,
+		GNOME_STOCK_BUTTON_OK,
+		GNOME_STOCK_BUTTON_CANCEL,
+		NULL);
+	v = gnome_dialog_run (GNOME_DIALOG (w));
+	if (v == 0){
+		/*
+		 * We do not want to be restarted by the session manager now
+		 */
+		gnome_client_set_restart_style (session_client, GNOME_RESTART_NEVER);
+		gmc_do_quit ();
+	}
+}
+
+static void
+do_desktop_rescan_devices (void)
+{
+	desktop_reload_icons (FALSE, 0, 0);
+}
+
 void configure_box (void);
 
 GtkCheckMenuItem *gnome_toggle_snap (void);
@@ -308,6 +341,8 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 	GNOMEUIINFO_ITEM_STOCK(N_("_Delete..."), N_("Delete files"), delete_cmd, GNOME_STOCK_PIXMAP_REMOVE),
         GNOMEUIINFO_ITEM_NONE(N_("_Move..."), N_("Rename or move files"), ren_cmd),
 	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_NONE(N_("Show directory sizes"), N_("Shows the disk space used by each directory"), dirsizes_cmd),
+	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_CLOSE_WINDOW_ITEM(gnome_close_panel, NULL),
         GNOMEUIINFO_END
 };
@@ -358,6 +393,9 @@ GnomeUIInfo gnome_panel_commands_menu [] = {
 #ifdef WITH_BACKGROUND
 	GNOMEUIINFO_ITEM_NONE( N_("_Background jobs..."),   N_("List of background operations"), jobs_cmd ),
 #endif
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_STOCK (N_("Exit"), N_("Terminates the file manager and the desktop"),
+				gnome_exit, GNOME_STOCK_PIXMAP_QUIT),
         GNOMEUIINFO_END
 };
 
@@ -368,12 +406,30 @@ GnomeUIInfo gnome_panel_about_menu [] = {
 	GNOMEUIINFO_END
 };
 
-GnomeUIInfo gnome_panel_menu [] = {
+GnomeUIInfo gnome_panel_desktop_menu [] = {
+	GNOMEUIINFO_ITEM_NONE (N_("Arrange Icons"), NULL, desktop_arrange_icons),
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan Mountable Devices"), NULL, desktop_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan Desktop"), NULL, do_desktop_rescan_devices),
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_panel_menu_with_desktop [] = {
         GNOMEUIINFO_MENU_FILE_TREE(gnome_panel_file_menu),
         GNOMEUIINFO_MENU_EDIT_TREE(gnome_panel_edit_menu),
         GNOMEUIINFO_SUBTREE(N_("_Layout"),gnome_panel_layout_menu),
         GNOMEUIINFO_SUBTREE(N_("_Commands"),gnome_panel_commands_menu),
-        GNOMEUIINFO_MENU_HELP_TREE(gnome_panel_about_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Desktop"), gnome_panel_desktop_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Help"), gnome_panel_about_menu),
+        GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_panel_menu_without_desktop [] = {
+        GNOMEUIINFO_MENU_FILE_TREE(gnome_panel_file_menu),
+        GNOMEUIINFO_MENU_EDIT_TREE(gnome_panel_edit_menu),
+        GNOMEUIINFO_SUBTREE(N_("_Layout"),gnome_panel_layout_menu),
+        GNOMEUIINFO_SUBTREE(N_("_Commands"),gnome_panel_commands_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Help"), gnome_panel_about_menu),
         GNOMEUIINFO_END
 };
 
@@ -404,55 +460,11 @@ panel_enter_event (GtkWidget *widget, GdkEvent *event, WPanel *panel)
 	if (get_current_panel () == panel)
 		return;
 	
-	set_new_current_panel (panel);
+	set_current_panel (panel);
 	dlg_select_widget (panel->widget.parent, panel);
 	send_message (panel->widget.parent, (Widget *) panel, WIDGET_FOCUS, 0);
 }
 
-struct _TbItems {
-	char *key, *text, *tooltip, *icon;
-	void (*cb) (GtkWidget *, void *);
-	GtkWidget *widget; /* will be filled in */
-};
-typedef struct _TbItems TbItems;
-
-void user_menu_cmd (void);
-
-static TbItems tb_items[] =
-{
-/* 1Help   2Menu   3View   4Edit   5Copy   6RenMov 7Mkdir  8Delete 9PullDn 10Quit */
-    {"F1", "Help", "Interactive help browser", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) help_cmd, 0},
-    {"F2", "Menu", "User actions", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) user_menu_cmd, 0},
-    {"F3", "View", "View file", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) view_panel_cmd, 0},
-    {"F4", "Edit", "Edit file", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) edit_panel_cmd, 0},
-    {"F5", "Copy", "Copy file or directory", GNOME_STOCK_MENU_COPY, (void (*) (GtkWidget *, void *)) copy_cmd, 0},
-    {"F6", "Move", "Rename or move a file or directory", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) ren_cmd, 0},
-    {"F7", "Mkdir", "Create directory", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) mkdir_panel_cmd, 0},
-    {"F8", "Dlete", "Delete file or directory", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) delete_cmd, 0},
-    {"F9", "Menu", "Pull down menu", GNOME_STOCK_MENU_BLANK, (void (*) (GtkWidget *, void *)) 0, 0},
-    {"F10", "Quit", "Close and exit", GNOME_STOCK_MENU_QUIT, (void (*) (GtkWidget *, void *)) 0, 0},
-    {0, 0, 0, 0, 0, 0}
-};
-
-static GtkWidget *create_toolbar (GtkWidget * window, GtkWidget *widget)
-{
-	GtkWidget *toolbar;
-	TbItems *t;
-	
-	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-	
-	for (t = &tb_items[0]; t->text; t++){
-		t->widget = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-						     t->text,
-					     t->tooltip,
-						     0,
-						     gnome_stock_pixmap_widget (window, t->icon),
-						     t->cb,
-						     t->cb ? widget : 0);
-	}
-	return toolbar;
-}
-
 static void
 create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
 {
@@ -486,6 +498,7 @@ create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
 		file2 = g_concat_dir_and_file (file, dirstruc->d_name);
 
 		if ((stat (file2, &filedata) != -1) && (S_ISREG (filedata.st_mode))){
+			char *path;
 			char *test;
 			
 			gde = gnome_desktop_entry_load (file2);
@@ -497,9 +510,10 @@ create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
 				continue;
 			}
 
-			if (!gnome_is_program_in_path (gde->tryexec)){
+			path = gnome_is_program_in_path (gde->tryexec);
+			g_free (path);
+			if (!path){
 				g_free (file2);
-				g_print ("yes!\n");
 				continue;
 			}
 			
@@ -568,18 +582,22 @@ create_container (Dlg_head *h, char *name, char *geometry)
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 	gnome_app_set_contents (GNOME_APP (app), vbox);
-	gnome_app_create_menus_with_data (GNOME_APP (app), gnome_panel_menu, panel);
+
+	if (desktop_wm_is_gnome_compliant == 1)
+		gnome_app_create_menus_with_data (GNOME_APP (app), gnome_panel_menu_without_desktop, panel);
+	else
+		gnome_app_create_menus_with_data (GNOME_APP (app), gnome_panel_menu_with_desktop, panel);
+	
 	create_new_menu (GNOME_APP (app), panel);
 
 	panel->ministatus = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER));
 	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(panel->ministatus));
 
-	gnome_app_install_menu_hints (GNOME_APP (app), gnome_panel_menu);
-	/*
-	 * I am trying to unclutter the screen, so this toolbar is gone now
-	 */
-/*	gnome_app_set_toolbar(GNOME_APP (app), GTK_TOOLBAR(create_toolbar(app, 0))); */
-
+	if (desktop_wm_is_gnome_compliant)
+		gnome_app_install_menu_hints (GNOME_APP (app), gnome_panel_menu_without_desktop);
+	else
+		gnome_app_install_menu_hints (GNOME_APP (app), gnome_panel_menu_with_desktop);
+	
 	gtk_signal_connect (GTK_OBJECT (app),
 			    "enter_notify_event",
 			    GTK_SIGNAL_FUNC (panel_enter_event),
@@ -616,7 +634,7 @@ new_panel_with_geometry_at (char *dir, char *geometry)
 	mc_chdir (dir);
 	panel = create_container (desktop_dlg, dir, geometry);
 	add_widget (desktop_dlg, panel);
-	set_new_current_panel (panel);
+	set_current_panel (panel);
 	x_flush_events ();
 
 	return panel;
