@@ -25,6 +25,7 @@
 #include "gnome-file-property-dialog.h"
 #define CLIST_FROM_SW(panel_list) GTK_CLIST (GTK_BIN (panel_list)->child)
 
+
 /* Flags for the popup menu entries.  They specify to which kinds of files an
  * entry is valid for.
  */
@@ -34,9 +35,9 @@ enum {
 	F_SYMLINK	= 1 << 2,	/* Applies only to symlinks */
 	F_SINGLE	= 1 << 3,	/* Applies only to a single file, not to multiple files */
 	F_NOTDIR	= 1 << 4,	/* Applies to non-directories */
-	F_DICON		= 1 << 5,	/* Applies only to desktop icons */
-	F_NOTDEV	= 1 << 6,	/* Applies to non-devices only (ie. reg, lnk, dir) */
-	F_ADVANCED	= 1 << 7        /* Only appears in advanced mode */
+	F_NOTDEV	= 1 << 5,	/* Applies to non-devices only (ie. reg, lnk, dir) */
+	F_ADVANCED	= 1 << 6,	/* Only appears in advanced mode */
+	F_MIME_ACTIONS	= 1 << 7	/* Special marker for the position of MIME actions */
 };
 
 /* typedefs */
@@ -46,8 +47,8 @@ struct action {
 	gpointer callback;	/* Callback for menu item */
 };
 
+
 /* Multiple File commands */
-static void panel_action_open_with (GtkWidget *widget, WPanel *panel);
 static void handle_open (GtkWidget *widget, WPanel *panel);
 static void handle_view (GtkWidget *widget, WPanel *panel);
 static void handle_view_unfiltered (GtkWidget *widget, WPanel *panel);
@@ -63,11 +64,6 @@ static void handle_hard_link (GtkWidget *widget, WPanel *panel);
 static void handle_symlink (GtkWidget *widget, WPanel *panel);
 static void handle_edit_symlink (GtkWidget *widget, WPanel *panel);
 
-/* Generic Options */
-static void handle_display_properties (GtkWidget *widget, WPanel *panel);
-static void handle_rescan (GtkWidget *widget, WPanel *panel);
-static void handle_arrange_icons (GtkWidget *widget, WPanel *panel);
-static void handle_logout (GtkWidget *widget, WPanel *panel);
 
 /* global vars */
 extern int we_can_afford_the_speed;
@@ -75,9 +71,10 @@ extern int we_can_afford_the_speed;
 static struct action file_actions[] = {
 	{ N_("Open"),			F_NOTDEV,				handle_open },
 	{ "",				F_NOTDEV,				NULL },
+	{ "",				F_MIME_ACTIONS,				NULL },
 	{ N_("Open with..."),		F_REGULAR | F_SINGLE, 			handle_open_with },
 	{ N_("View"),			F_REGULAR | F_SINGLE, 			handle_view },
-	{ N_("View Unfiltered"),	F_REGULAR | F_ADVANCED | F_SINGLE,	handle_view_unfiltered },  
+	{ N_("View Unfiltered"),	F_REGULAR | F_ADVANCED | F_SINGLE,	handle_view_unfiltered },
 	{ N_("Edit"),			F_REGULAR | F_SINGLE, 			handle_edit },
 	{ "",				F_REGULAR | F_SINGLE, 			NULL },
 	{ N_("Copy..."),		F_ALL, 					handle_copy },
@@ -170,11 +167,103 @@ fill_menu (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo, int pos)
 	gnome_app_fill_menu_custom (menu_shell, uiinfo, &uibdata, NULL, FALSE, pos);
 }
 
+/* Convenience function to free something when an object is destroyed */
+static void
+free_on_destroy (GtkObject *object, gpointer data)
+{
+	g_free (data);
+}
 
-/* The context menu: text displayed, condition that must be met and the routine
- * that gets invoked upon activation.
+/* Callback for MIME-based actions */
+static void
+mime_action_callback (GtkWidget *widget, gpointer data)
+{
+	char *filename;
+	char *key;
+	char *mime_type;
+	char *value;
+
+	filename = data;
+	key = gtk_object_get_user_data (GTK_OBJECT (widget));
+
+	g_assert (filename != NULL);
+	g_assert (key != NULL);
+
+	mime_type = gnome_mime_type_or_default (filename, NULL);
+	g_assert (mime_type != NULL);
+
+	value = gnome_mime_get_value (mime_type, key);
+	exec_extension (filename, value, NULL, NULL, 0);
+}
+
+/* Creates the menu items for actions based on the MIME type of the selected
+ * file in the panel.
  */
+static int
+create_mime_actions (GtkWidget *menu, WPanel *panel, int pos)
+{
+	char *full_name;
+	const char *mime_type;
+	GList *keys, *l;
+	GnomeUIInfo uiinfo[] = {
+		{ 0 },
+		GNOMEUIINFO_END
+	};
 
+	full_name = g_concat_dir_and_file (panel->cwd, panel->dir.list[panel->selected].fname);
+	mime_type = gnome_mime_type_or_default (full_name, NULL);
+	g_free (full_name);
+
+	if (!mime_type)
+		return pos;
+
+	keys = gnome_mime_get_keys (mime_type);
+	for (l = keys; l; l = l->next) {
+		char *key;
+		char *str;
+
+		str = key = l->data;
+
+		if (strncmp (key, "open.", 5) != 0)
+			continue;
+
+		str += 5;
+		while (*str && *str != '.')
+			str++;
+
+		if (*str)
+			str++;
+
+		if (!*str)
+			continue;
+
+		/* Create the item for that entry */
+
+		uiinfo[0].type = GNOME_APP_UI_ITEM;
+		uiinfo[0].label = str;
+		uiinfo[0].hint = NULL;
+		uiinfo[0].moreinfo = mime_action_callback;
+		uiinfo[0].user_data = full_name;
+		uiinfo[0].unused_data = NULL;
+		uiinfo[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
+		uiinfo[0].pixmap_info = NULL;
+		uiinfo[0].accelerator_key = 0;
+		uiinfo[0].ac_mods = 0;
+		uiinfo[0].widget = NULL;
+
+		fill_menu (GTK_MENU_SHELL (menu), uiinfo, pos++);
+		gtk_object_set_user_data (GTK_OBJECT (uiinfo[0].widget), key);
+
+		/* Remember to free this memory */
+
+		gtk_signal_connect (GTK_OBJECT (uiinfo[0].widget), "destroy",
+				    (GtkSignalFunc) free_on_destroy,
+				    full_name);
+	}
+
+	g_list_free (keys);
+	return pos;
+}
 
 /* Creates the menu items for the standard actions.  Returns the position at
  * which additional menu items should be inserted.
@@ -192,9 +281,17 @@ create_actions (GtkWidget *menu, gint flags, WPanel *panel)
 	pos = 0;
 
 	for (action = file_actions; action->text; action++) {
+		/* Insert the MIME actions if appropriate */
+		if ((action->flags & F_MIME_ACTIONS) && (flags & F_SINGLE)) {
+			pos = create_mime_actions (menu, panel, pos);
+			continue;
+		}
+
+		/* Filter the actions that are not appropriate */
 		if ((action->flags & flags) != action->flags)
 			continue;
 
+		/* Create the menu item for this action */
 		if (action->text[0]) {
 			uiinfo[0].type = GNOME_APP_UI_ITEM;
 			uiinfo[0].label = _(action->text);
@@ -308,28 +405,13 @@ gpopup_do_popup2 (GdkEventButton *event, WPanel *panel)
 	return i;
 }
 
-static void
-panel_action_open_with (GtkWidget *widget, WPanel *panel)
-{
-	char *command;
-	
-	command = input_expand_dialog (_(" Open with..."),
-				       _("Enter extra arguments:"),
-				       panel->dir.list [panel->selected].fname);
-	if (!command)
-		return;
-
-	execute (command);
-	g_free (command);
-}
-
 static void
 handle_open (GtkWidget *widget, WPanel *panel)
 {
 	if (do_enter (panel))
 		return;
 
-	panel_action_open_with (widget, panel);
+	handle_open_with (widget, panel);
 }
 
 static void
@@ -373,14 +455,13 @@ static void
 handle_properties (GtkWidget *widget, WPanel *panel)
 {
 	gint retval;
-	file_entry *fe = &panel->dir.list [panel->selected];
-	char *full_name = concat_dir_and_file (panel->cwd, fe->fname);
+	file_entry *fe;
+	char *full_name;
 	GtkWidget *dlg;
 
-#if 0
-	if (item_properties (GTK_WIDGET (CLIST_FROM_SW (panel->list)), full_name, NULL) != 0)
-		reread_cmd ();
-#endif
+	fe = &panel->dir.list [panel->selected];
+	full_name = concat_dir_and_file (panel->cwd, fe->fname);
+
 	dlg = gnome_file_property_dialog_new (full_name, we_can_afford_the_speed);
 	gnome_dialog_set_parent (GNOME_DIALOG (dlg),
 				 GTK_WINDOW (gtk_widget_get_toplevel (panel->ministatus)));
@@ -399,7 +480,7 @@ handle_open_with (GtkWidget *widget, WPanel *panel)
 {
 	char *command;
 
-	command = input_expand_dialog (_(" Open with..."),
+	command = input_expand_dialog (_("Open with"),
 				       _("Enter extra arguments:"),
 				       panel->dir.list [panel->selected].fname);
 	if (!command)
@@ -428,29 +509,8 @@ handle_edit_symlink (GtkWidget *widget, WPanel *panel)
 	edit_symlink_cmd ();
 }
 
-static void
-handle_display_properties (GtkWidget *widget, WPanel *panel)
-{
-	/* FIXME */
-	g_warning ("FIXME: implement popup_handle_display_properties()");
-}
-
 static void
 handle_rescan (GtkWidget *widget, WPanel *panel)
 {
 	reread_cmd ();
 }
-
-static void
-handle_arrange_icons (GtkWidget *widget, WPanel *panel)
-{
-	/* FIXME */
-	g_warning ("FIXME: implement popup_handle_arrange_icons()");
-}
-
-static void
-handle_logout (GtkWidget *widget, WPanel *panel)
-{
-	/* FIXME */
-	g_warning ("FIXME: implement popup_handle_logout()");
-}
