@@ -232,7 +232,7 @@ panel_fill_panel_icons (WPanel *panel)
 		file_entry *fe = &panel->dir.list [i];
 		int p;
 		
-		image = gicon_get_icon_for_file (panel->cwd, fe);
+		image = gicon_get_icon_for_file (panel->cwd, fe, TRUE);
 		p = gnome_icon_list_append_imlib (icons, image, fe->fname);
 		if (fe->f.marked)
 			gnome_icon_list_select_icon (icons, p);
@@ -798,14 +798,20 @@ panel_icon_list_drag_data_received (GtkWidget          *widget,
 		free_file = FALSE;
 		free_fe = TRUE;
 	} else {
-		file = g_concat_dir_and_file (panel->cwd, panel->dir.list[idx].fname);
 		fe = &panel->dir.list[idx];
 
-		free_file = TRUE;
+		if (S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir){
+			file = g_concat_dir_and_file (panel->cwd, panel->dir.list[idx].fname);
+			free_file = TRUE;
+		} else {
+			file = panel->cwd;
+			free_file = FALSE;
+		}
+		
 		free_fe = FALSE;
 	}
 
-	reload = gdnd_perform_drop (context, selection_data, fe, file);
+	reload = gdnd_perform_drop (context, selection_data, file, fe);
 
 	if (free_file)
 		g_free (file);
@@ -853,14 +859,20 @@ panel_clist_drag_data_received (GtkWidget          *widget,
 	} else {
 		g_assert (row < panel->count);
 
-		file = g_concat_dir_and_file (panel->cwd, panel->dir.list[row].fname);
 		fe = &panel->dir.list[row];
 
-		free_file = TRUE;
+		if (S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir){
+			file = g_concat_dir_and_file (panel->cwd, panel->dir.list[row].fname);
+			free_file = TRUE;
+		} else {
+			file = panel->cwd;
+			free_file = FALSE;
+		}
+		
 		free_fe = FALSE;
 	}
 
-	reload = gdnd_perform_drop (context, selection_data, fe, file);
+	reload = gdnd_perform_drop (context, selection_data, file, fe);
 
 	if (free_file)
 		g_free (file);
@@ -908,7 +920,7 @@ panel_tree_drag_data_received (GtkWidget          *widget,
 	if (!fe)
 		return; /* eeeek */
 
-	gdnd_perform_drop (context, selection_data, fe, path);
+	gdnd_perform_drop (context, selection_data, path, fe);
 
 	file_entry_free (fe);
 	g_free (path);
@@ -1117,6 +1129,7 @@ panel_clist_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gin
 				       FALSE,
 				       source_widget != NULL,
 				       source_widget == widget,
+				       panel->cwd,
 				       fe,
 				       fe ? fe->f.marked : FALSE);
 
@@ -1232,6 +1245,7 @@ panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x,
 				       FALSE,
 				       source_widget != NULL,
 				       source_widget == widget,
+				       panel->cwd,
 				       fe,
 				       fe ? fe->f.marked : FALSE);
 
