@@ -139,11 +139,6 @@ typedef struct {
     void *ministatus;
     void *tree_scrolled_window;
 
-#ifdef NEW_GNOME_LIBS
-#else
-    void *scrollbar;
-#endif
-	
     void *filter_w;		/* A WInput* */
     void *current_dir;		/* A WInput* */
     int estimated_total;
@@ -164,7 +159,6 @@ typedef struct {
     int drag_motion_y;
     void *panel_listbox;	/* container for the list */
 #endif
-    void *corbadat;
 } WPanel;
 
 WPanel *panel_new (char *panel_name);
