@@ -132,6 +132,7 @@ void change_nroff       (WView *view);
 void set_monitor        (WView *view, int set_on);
 void view_move_forward  (WView *view, int i);
 void view_move_backward (WView *view, int i);
+void view_add_one_vline (void);
 #endif
 
 /* Command: view a file, if _command != NULL we use popen on _command */
@@ -150,7 +151,7 @@ extern int altered_hex_mode;
 extern int altered_magic_flag;
 extern int altered_nroff_flag;
 
-void view_adjust_size ();
+void view_adjust_size (Dlg_head *);
 
 /* A node for building a change list on change_list */
 struct hexedit_change_node {
